@@ -6,7 +6,7 @@ from tensorflow.python.ops import array_ops, control_flow_ops, math_ops, state_o
 from tensorflow.python.util.tf_export import keras_export
 from tensorflow.python.keras import backend as K
 from .utils import _init_weight_decays, _apply_weight_decays, _check_args
-from .utils import _update_t_cur_eta_t_apply_lr_mult
+from .utils import _update_t_cur_eta_t_v2, _apply_lr_multiplier, _set_autorestart
 from .utils import K_eval as KE
 
 
@@ -51,7 +51,6 @@ class AdamW(OptimizerV2):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -62,6 +61,13 @@ class AdamW(OptimizerV2):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [2]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [2]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -88,11 +94,12 @@ class AdamW(OptimizerV2):
     """
     def __init__(self, learning_rate=0.001, beta_1=0.9, beta_2=0.999,
                  epsilon=None, decay=0., amsgrad=False,
-                 model=None, zero_penalties=True, batch_size=32,
+                 model=None, zero_penalties=True,
                  total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
-                 eta_min=0, eta_max=1, t_cur=0, name="AdamW", **kwargs):
+                 weight_decays=None, autorestart=None,
+                 init_verbose=True, eta_min=0, eta_max=1, t_cur=0,
+                 name="AdamW", **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
                                                 weight_decays)
@@ -108,7 +115,6 @@ class AdamW(OptimizerV2):
         self.eta_max = K.constant(eta_max, name='eta_max')
         self.eta_t = K.variable(eta_t, dtype='float32', name='eta_t')
         self.t_cur = K.variable(t_cur, dtype='int64', name='t_cur')
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.lr_multipliers = lr_multipliers
@@ -118,6 +124,7 @@ class AdamW(OptimizerV2):
         self.epsilon = epsilon or backend_config.epsilon()
         self.amsgrad = amsgrad
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = kwargs.get('lr', learning_rate)  # to print lr_mult setup
         self._updates_processed = 0  # to track num calls to '_resource_apply_...'
@@ -148,11 +155,9 @@ class AdamW(OptimizerV2):
         epsilon_t = ops.convert_to_tensor(self.epsilon, var_dtype)
 
         lr_t = lr_t * math_ops.sqrt(1 - beta_2_power) / (1 - beta_1_power)
-
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         m_t = state_ops.assign(m,
                                beta_1_t * m + (1.0 - beta_1_t) * grad,
@@ -175,9 +180,13 @@ class AdamW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update, m_t, v_t]
         if iteration_done:
@@ -201,11 +210,9 @@ class AdamW(OptimizerV2):
         epsilon_t = ops.convert_to_tensor(self.epsilon, var_dtype)
 
         lr_t = lr_t * math_ops.sqrt(1 - beta_2_power) / (1 - beta_1_power)
-
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         m_scaled_g_values = grad * (1 - beta_1_t)
         m_t = state_ops.assign(m, m * beta_1_t, use_locking=self._use_locking)
@@ -231,9 +238,13 @@ class AdamW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update, m_t, v_t]
         if iteration_done:
@@ -263,10 +274,10 @@ class AdamW(OptimizerV2):
             'beta_2': self._serialize_hyperparameter('beta_2'),
             'epsilon': self.epsilon,
             'amsgrad': self.amsgrad,
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K_eval(self.t_cur)),
             'eta_t': float(K_eval(self.eta_t)),
             'eta_min': float(K_eval(self.eta_min)),
@@ -306,7 +317,6 @@ class NadamW(OptimizerV2):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -317,6 +327,13 @@ class NadamW(OptimizerV2):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [3]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [3]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -344,10 +361,10 @@ class NadamW(OptimizerV2):
              (https://arxiv.org/abs/1711.05101)
     """
     def __init__(self, learning_rate=0.001, beta_1=0.9, beta_2=0.999,
-                 epsilon=1e-7, model=None, zero_penalties=True, batch_size=32,
+                 epsilon=1e-7, model=None, zero_penalties=True,
                  total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
+                 weight_decays=None, autorestart=None, init_verbose=True,
                  eta_min=0, eta_max=1, t_cur=0, name="NadamW", **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
@@ -374,7 +391,6 @@ class NadamW(OptimizerV2):
         self.eta_max = K.constant(eta_max, name='eta_max')
         self.eta_t = K.variable(eta_t, dtype='float32', name='eta_t')
         self.t_cur = K.variable(t_cur, dtype='int64', name='t_cur')
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.lr_multipliers = lr_multipliers
@@ -383,6 +399,7 @@ class NadamW(OptimizerV2):
         self.use_cosine_annealing = use_cosine_annealing
         self.epsilon = epsilon or backend_config.epsilon()
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = kwargs.get('lr', learning_rate)  # to print lr_mult setup
         self._updates_processed = 0  # to track num calls to '_resource_apply_...'
@@ -420,9 +437,8 @@ class NadamW(OptimizerV2):
         decay_base = math_ops.cast(0.96, var_dtype)
 
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         # Due to the recommendations in [2], i.e. warming momentum schedule
         momentum_cache_t = beta_1_t * (1. - 0.5 * (
@@ -455,9 +471,13 @@ class NadamW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update, m_t, v_t]
         if iteration_done:
@@ -479,9 +499,8 @@ class NadamW(OptimizerV2):
         decay_base = math_ops.cast(0.96, var_dtype)
 
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         momentum_cache_t = beta_1_t * (1. - 0.5 * (
             math_ops.pow(decay_base, self._initial_decay * local_step)))
@@ -524,9 +543,13 @@ class NadamW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update, m_t_bar, v_t]
         if iteration_done:
@@ -543,10 +566,10 @@ class NadamW(OptimizerV2):
             'beta_1': self._serialize_hyperparameter('beta_1'),
             'beta_2': self._serialize_hyperparameter('beta_2'),
             'epsilon': self.epsilon,
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K_eval(self.t_cur)),
             'eta_t': float(K_eval(self.eta_t)),
             'eta_min': float(K_eval(self.eta_min)),
@@ -580,7 +603,6 @@ class SGDW(OptimizerV2):
             extracts weight penalties from layers, and overrides `weight_decays`.
         zero_penalties: bool. If True and `model` is passed, will zero weight
             penalties (loss-based). (RECOMMENDED; see README "Use guidelines").
-        batch_size:       int >= 1. Train input batch size; used for normalization
         total_iterations: int >= 0. Total expected iterations / weight updates
                           throughout training, used for normalization; <1>
         lr_multipliers:   dict / None. Name-value pairs specifying per-layer lr
@@ -591,6 +613,13 @@ class SGDW(OptimizerV2):
         use_cosine_annealing: bool. If True, multiplies lr each train iteration
                               as a function of eta_min, eta_max, total_iterations,
                               and t_cur (current); [2]-Appendix, 2
+        autorestart: bool / None. If True, will automatically do Warm Restarts
+                     by resetting `t_cur=0` after `total_iterations`. If None,
+                     will default to same as `use_cosine_annealing`. If True
+                     but `use_cosine_annealing` is False, will raise ValueError.
+                     Note: once optimizer is built (happens on first model fit),
+                     changing `autorestart` has no effect; optimizer needs to be
+                     re-built.
         eta_min, eta_max: int, int. Min & max values of cosine annealing
                           lr multiplier; [2]-Appendix, 2
         t_cur: int. Value to initialize t_cur to - used for 'warm restarts'.
@@ -617,10 +646,10 @@ class SGDW(OptimizerV2):
          (https://arxiv.org/abs/1711.05101)
     """
     def __init__(self, learning_rate=0.01, momentum=0.0, nesterov=False,
-                 model=None, zero_penalties=True, batch_size=32,
+                 model=None, zero_penalties=True,
                  total_iterations=0, total_iterations_wd=None,
                  use_cosine_annealing=False, lr_multipliers=None,
-                 weight_decays=None, init_verbose=True,
+                 weight_decays=None, autorestart=None, init_verbose=True,
                  eta_min=0, eta_max=1, t_cur=0, name="SGDW", **kwargs):
         if total_iterations > 1:
             weight_decays = _init_weight_decays(model, zero_penalties,
@@ -643,7 +672,6 @@ class SGDW(OptimizerV2):
         self.eta_max = K.constant(eta_max, name='eta_max')
         self.eta_t = K.variable(eta_t, dtype='float32', name='eta_t')
         self.t_cur = K.variable(t_cur, dtype='int64', name='t_cur')
-        self.batch_size = batch_size
         self.total_iterations = total_iterations
         self.total_iterations_wd = total_iterations_wd or total_iterations
         self.lr_multipliers = lr_multipliers
@@ -651,6 +679,7 @@ class SGDW(OptimizerV2):
         self.init_verbose = init_verbose
         self.use_cosine_annealing = use_cosine_annealing
 
+        _set_autorestart(self, autorestart, use_cosine_annealing)
         _check_args(self, total_iterations, use_cosine_annealing, weight_decays)
         self._init_lr = kwargs.get('lr', learning_rate)  # to print lr_mult setup
         self._updates_processed = 0  # to track num calls to '_resource_apply_...'
@@ -667,9 +696,8 @@ class SGDW(OptimizerV2):
         lr_t = self._decayed_lr(var_dtype)
 
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         if self._momentum:
             momentum = array_ops.identity(self._get_hyper('momentum', var_dtype))
@@ -690,9 +718,13 @@ class SGDW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update]
         if self._momentum:
@@ -708,9 +740,8 @@ class SGDW(OptimizerV2):
         lr_t = self._decayed_lr(var_dtype)
 
         # Learning rate multipliers
-        # Cosine annealing
-        (iteration_done, t_cur_update, eta_t_update
-         ) = _update_t_cur_eta_t_apply_lr_mult(self, lr_t, var)
+        if self.lr_multipliers is not None:
+            lr_t = _apply_lr_multiplier(self, lr_t, var)
 
         if self._momentum:
             momentum = array_ops.identity(self._get_hyper('momentum', var_dtype))
@@ -731,9 +762,13 @@ class SGDW(OptimizerV2):
         if var.name in self.weight_decays.keys():
             var_t = _apply_weight_decays(self, var, var_t)
 
+        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
+
+        # Cosine annealing
+        (iteration_done, t_cur_update, eta_t_update
+         ) = _update_t_cur_eta_t_v2(self, lr_t, var)
         if iteration_done and not self._init_notified:
             self._init_notified = True
-        var_update = state_ops.assign(var, var_t, use_locking=self._use_locking)
 
         updates = [var_update]
         if self._momentum:
@@ -751,10 +786,10 @@ class SGDW(OptimizerV2):
             "decay": self._serialize_hyperparameter("decay"),
             "momentum": self._serialize_hyperparameter("momentum"),
             "nesterov": self.nesterov,
-            'batch_size': int(self.batch_size),
             'total_iterations': int(self.total_iterations),
             'weight_decays': self.weight_decays,
             'use_cosine_annealing': self.use_cosine_annealing,
+            'autorestart': self.autorestart,
             't_cur': int(K_eval(self.t_cur)),
             'eta_t': float(K_eval(self.eta_t)),
             'eta_min': float(K_eval(self.eta_min)),
